@@ -28,6 +28,7 @@
 #include <gedit/gedit-window.h>
 #include <gedit/gedit-view.h>
 #include <gedit/gedit-tab.h>
+#include <gedit/gedit-utils.h>
 
 #include <gconf/gconf-client.h>
 
@@ -53,6 +54,7 @@ GEDIT_PLUGIN_REGISTER_TYPE (GeditDrawspacesPlugin, gedit_drawspaces_plugin)
 struct _GeditDrawspacesPluginPrivate
 {
 	GConfClient *gconf_client;
+	guint connection_id;
 	
 	GtkSourceDrawSpacesFlags flags;
 };
@@ -170,24 +172,37 @@ gedit_drawspaces_plugin_init (GeditDrawspacesPlugin *plugin)
 			      GCONF_CLIENT_PRELOAD_ONELEVEL,
 			      NULL);
 			      
-	gconf_client_notify_add (plugin->priv->gconf_client,
-				 GCONF_KEY_BASE,
-				 on_gconf_notify,
-				 plugin, NULL, NULL);
+	plugin->priv->connection_id = gconf_client_notify_add (plugin->priv->gconf_client,
+							       GCONF_KEY_BASE,
+							       on_gconf_notify,
+							       plugin, NULL, NULL);
 }
 
 static void
-gedit_drawspaces_plugin_finalize (GObject *object)
+gedit_drawspaces_plugin_dispose (GObject *object)
 {
 	GeditDrawspacesPlugin *plugin = GEDIT_DRAWSPACES_PLUGIN (object);
 
-	gedit_debug_message (DEBUG_PLUGINS, "GeditDrawspacesPlugin finalizing");
+	gedit_debug_message (DEBUG_PLUGINS, "GeditDrawspacesPlugin disposing");
 
-	gconf_client_suggest_sync (plugin->priv->gconf_client, NULL);
+	if (plugin->priv->connection_id != 0)
+	{
+		gconf_client_notify_remove (plugin->priv->gconf_client,
+					    plugin->priv->connection_id);
+		
+		plugin->priv->connection_id = 0;
+	}
+	
+	if (plugin->priv->gconf_client != NULL)
+	{
+		gconf_client_suggest_sync (plugin->priv->gconf_client, NULL);
 
-	g_object_unref (G_OBJECT (plugin->priv->gconf_client));
-
-	G_OBJECT_CLASS (gedit_drawspaces_plugin_parent_class)->finalize (object);
+		g_object_unref (G_OBJECT (plugin->priv->gconf_client));
+		
+		plugin->priv->gconf_client = NULL;
+	}
+	
+	G_OBJECT_CLASS (gedit_drawspaces_plugin_parent_class)->dispose (object);
 }
 
 static void
@@ -274,7 +289,7 @@ get_config_options (WindowData *data,
 	gboolean tabs, spaces, newline, nbsp;
 
 	data->enable = get_gconf_value_with_default (plugin, GCONF_KEY_ENABLE,
-						     FALSE);
+						     TRUE);
 
 	tabs = get_gconf_value_with_default (plugin, GCONF_KEY_DRAW_TABS,
 					     TRUE);
@@ -302,8 +317,7 @@ static void
 impl_activate (GeditPlugin *plugin,
 	       GeditWindow *window)
 {
-	GList *views, *l;
-	GeditDrawspacesPlugin *ds_plugin = GEDIT_DRAWSPACES_PLUGIN (plugin);
+	GeditDrawspacesPlugin *ds_plugin;
 	GtkUIManager *manager;
 	GError *error = NULL;
 	GtkAction *action;
@@ -312,6 +326,8 @@ impl_activate (GeditPlugin *plugin,
 
 	gedit_debug (DEBUG_PLUGINS);
 
+	ds_plugin = GEDIT_DRAWSPACES_PLUGIN (plugin);
+
 	data = g_slice_new (WindowData);
 	action_data = g_slice_new (ActionData);
 	
@@ -381,10 +397,10 @@ impl_deactivate	(GeditPlugin *plugin,
 	
 	manager = gedit_window_get_ui_manager (window);
 	
-	ds_plugin->priv->flags = 0;
-	draw_spaces (ds_plugin);
+	data->enable = FALSE;
+	draw_spaces_in_window (window, ds_plugin);
 	
-	g_signal_handlers_disconnect_by_func (window, tab_added_cb, NULL);
+	g_signal_handlers_disconnect_by_func (window, tab_added_cb, ds_plugin);
 	
 	gtk_ui_manager_remove_ui (manager, data->ui_id);
 	gtk_ui_manager_remove_action_group (manager, data->action_group);
@@ -630,7 +646,7 @@ gedit_drawspaces_plugin_class_init (GeditDrawspacesPluginClass *klass)
 
 	g_type_class_add_private (object_class, sizeof (GeditDrawspacesPluginPrivate));
 
-	object_class->finalize = gedit_drawspaces_plugin_finalize;
+	object_class->dispose = gedit_drawspaces_plugin_dispose;
 
 	plugin_class->activate = impl_activate;
 	plugin_class->deactivate = impl_deactivate;
